$(document).ready(function(){

  var utils = window.UE.utils;

  function toTextNode(node, text) {
      node.data = text || '';
      node.children = undefined;
      node.type = 'text';
      node.tagName = undefined;
      node.attrs = {};
      return node;
  }

  function toEmptyNode(node) {
      toTextNode(node, '');
  }

  function clearNodeStyle(root) {
      if (root.type == 'element') {
          if (root.children && root.children.length) {
              utils.each(root.children, function(item) {
                  clearNodeStyle(item);
              });
          }
          root.setAttr('style', '');
      }
  }

  function clearTextNbsp(root) {
      if (root.type == 'element') {
          if (root.children && root.children.length) {
              utils.each(root.children, function(item) {
                  clearTextNbsp(item);
              });
          }
      } else if (root.type == 'text') {
          root.data = root.data.replace(/&[a-zA-Z]+;/g, '');
      }
  }

  function clearNodeTagName(root, tagName) {
      if (root.type === 'element') {
          if (root.children && root.children.length) {
              utils.each(root.children, function(item) {
                  clearNodeTagName(item, tagName);
              });
          }
          if (root.tagName === tagName) {
              toTextNode(root);
          }
      }
  }

  function clearNodeWithQQVideo(root) {
      if (root.type == 'element') {
          if (root.children && root.children.length) {
              utils.each(root.children, function(item) {
                  clearNodeWithQQVideo(item);
              });
          }
          if (root.tagName === 'iframe' && root.attrs.src && root.attrs.src.indexOf('v.qq.com/iframe')) {
              toTextNode(root);
          }
      }
  }

  function clearNodeIframeBackgroundImg(root) {
      if (root.type == 'element') {
          if (root.children && root.children.length) {
              utils.each(root.children, function(item) {
                  clearNodeIframeBackgroundImg(item);
              });
          }
          if (root.tagName === 'iframe' && root.attrs.class === 'video_iframe') {
              root.attrs.style = '';
          }
      }
  }

  function hasClass(node, name) {
    if (node && node.attrs && node.attrs.class && node.attrs.class.indexOf(name) > -1) {
      return true;
    }
    return false;
  }

  function makeNode(name, attrs) {
    return new  UE.uNode({
      type: "element",
      tagName: name,
      children: [],
      attrs: attrs
    });
  }

  var InputFilterMap = {
    'wrap-iframe': function(root) {
          var iframes = root.getNodesByTagName('iframe');
          utils.each(iframes, function (item, i) {
            var parent = item.parentNode;
            console.log('parent:::', parent, item);
            if (!parent) {
              return;
            }
            if (hasClass(parent, 'xb_iframe_wrap') || hasClass(parent, 'video_iframe_wrap')) {
              return;
            }
            if (!hasClass(item, 'video_iframe') && !hasClass(item, 'audio_iframe') && !hasClass(item, 'qqmusic_iframe')) {
              return;
            }
            if (parent.type == 'root') {
              var html = item.toHtml();
              item.tagName = 'p';
              item.attrs = {
                class: hasClass(item, 'video_iframe') ? 'video_iframe_wrap' : 'xb_iframe_wrap'
              };
              item.innerHTML(html);
              return;
            }
            if (hasClass(item, 'video_iframe')) {
              parent.innerHTML('<p class="video_iframe_wrap">' + parent.innerHTML() + '</p>');
            } else {
              parent.innerHTML('<p class="xb_iframe_wrap">' + parent.innerHTML() + '</p>');
            }
          });
      },
      'remove-image-webp': function (root) {
          var imgs = root.getNodesByTagName('img');
          utils.each(imgs, function (item) {
             var src = item.getAttr('src');
             if (src) {
                 src = src.replace(/tp=webp[&]?/ig, '');
                 const type = src.match(/wx_fmt=(\w*)/);

                 item.setAttr('src', src);
                 item.setAttr('data-type', type ? type[1] : 'png');
                 item.setAttr('data-w', 'auto');
                 item.setAttr('data-ratio', '0.66796875');
             }
         });
      },
  };

  window.UE.plugin.register('content-filter', function () {
      return {
          bindEvents: {
              ready: function () {

              }
          },

          outputRule: function () {

          },

          inputRule: function (root) {
              var me = this;
              for(var item in InputFilterMap) {
                InputFilterMap[item](root);
              }
          },

          commands: {

          }
      };
  });



});


function setupBridge(callback) {
  if (window.WebViewJavascriptBridge) { return callback(WebViewJavascriptBridge); }
  if (window.WVJBCallbacks) { return window.WVJBCallbacks.push(callback); }
  window.WVJBCallbacks = [callback];
  var WVJBIframe = document.createElement('iframe');
  WVJBIframe.style.display = 'none';
  WVJBIframe.src = 'wvjbscheme://__BRIDGE_LOADED__';
  document.documentElement.appendChild(WVJBIframe);
  setTimeout(function() { document.documentElement.removeChild(WVJBIframe) }, 10)
}

var CallApp = function(name, data, cb){
  console.log('call app:', name, data)
  setupBridge(function(bridge){
    bridge.callHandler(name, JSON.stringify(data), cb || function(){});
  });
}

$().ready(function() {
  // 编辑器交互接口
  var Editor = window.UE.getEditor('J_editor', {
    initialFrameHeight: 300,
    focus: true,
    allowDivTransToP: true,
    enableAutoSave: false,
    saveInterval: 10 * 24 * 60 * 1000,
    imageScaleEnabled: false,
    showTitle: false,
    isShow: true,
    disabledTableInTable: false,
    editorFor: 'default',
    canRemoveStylePickerColor: false,
    pasteUploadImage: false,
    iframeCssExtendUrl: 'editor.css'
  });


  Editor.on('selectionchange', function(e, r){
    var state = {
            undo: Editor.queryCommandState('undo'),
            redo: Editor.queryCommandState('redo'),
      fontsize: Editor.queryCommandValue('fontsize'),
      forecolor: Editor.queryCommandValue('forecolor'),
      bold: Editor.queryCommandState('bold'),
      italic: Editor.queryCommandState('italic'),
      underline: Editor.queryCommandState('underline'),
      strikethrough: Editor.queryCommandState('strikethrough'),
      indent: Editor.queryCommandState('indent'),
      imagefloat: Editor.queryCommandValue('imagefloat'),
      backcolor: Editor.queryCommandValue('backcolor'),
      justify: Editor.queryCommandValue('justify'),
      blank: Editor.selection.getRange().collapsed
    };
    var link = Editor.queryCommandValue('link');
    if (link) {
      state.link = link.href;
    }
    Editor._tempRange = null;
    CallApp('SetCommandValue', state);
  });

  document.addEventListener('paste', function(e){
    var b = e.clipboardData.getData('text/plain');
    if (b && b.match(/<\/\w+>/)) {
        e.preventDefault();
        Editor.execCommand('insertHtml', b.replace(/[\r\n]/g, '<br/>'));
    } else {
        Editor.fireEvent('contentChange');
    }
  });

  Editor.on('contentChange', function() {
    console.log('content change...');
    CallApp('SaveContent', Editor.getContent());
  });

  // 替换图片url
  Editor._replaceImgSrc = function(url){
    if (!Editor._clickTarget) {
      return;
    }
    Editor._clickTarget.attr('src', url).attr('_src', url);
    Editor.fireEvent('contentChange');
    // var html = _clickTarget[0].outerHTML;
    // Editor.selection.getRange().setStart(Editor._clickTarget[0], 0).setCursor(false, true);
    // Editor._clickTarget.remove();
    // Editor.execCommand('inserthtml', html.replace(/src="([^"]*)"/ig, 'src="'+url+ '"'));
  };

  // 视频模式
  Editor._onlyVideo = function(type) {
    var frame = $('.video_iframe').eq(0);
    if (!frame.length) {
      alert('没有找到视频内容!');
      return;
    }
    // var videoTemplate = {
    //     type0: '<div>$html' +
    //         '<strong style="width: 500px;height: 42px;margin-top: -42px;' +
    //         'color: rgb(255, 255, 255);display: block;text-align: center;">' +
    //         '关注我们查看更多视频</strong></div>' +
    //         '<section style="margin-top:-58px;"></section>',
    //     type1: '$html'
    // };
    // 修改iframe高度和宽度
    frame.attr('height', 310).attr('width', 500);
    // 转换视频路径，去除其他参数
    var match = $('#JAPPEditor').html().match(/<iframe[^>]*?vid=([\w]+)[^>]*>/im);
    if (!match) {
      alert('没有视频vid!');
      return;
    }
    var src;
    if (type == 0) {
      frame.attr('height', 194).attr('width', 345);
       src = 'https://v.qq.com/iframe/player.html?vid='+match[1]+'&width=345&height=194&auto=0;'
    } else {

      frame.attr('data-vidtype', 2);
      frame.attr('data-ratio', 1.76);
      frame.attr('data-w', 848);
      frame.attr('allowfullscreen', '');
      src = 'https://v.qq.com/iframe/preview.html?auto=0&vid=' + match[1];

    }
    frame.attr('src', src).attr('data-src', src);
    // var html = videoTemplate['type'+type].replace('$html', );
    console.log(html);
    var html = frame[0].outerHTML;
    Editor.setContent(html);
  };

  Editor._removeMpcpc = function() {
    var html = Editor.getContent();
    html = html.replace(/<mpcpc .*?<\/mpcpc>/g, '');
    Editor.setContent(html);
  };

  // 删除选中
  Editor._removeClickTarget = function(){
    if (!Editor._clickTarget) {
      return;
    }
    if (Editor._clickTarget[0].tagName.toLocaleLowerCase() == 'img') {
      var parent = Editor._clickTarget.parent();
      if (parent[0].tagName.toLocaleLowerCase() == 'a') {
        Editor._clickTarget = parent;
      }
    }
    Editor.selection.getRange().setStart(Editor._clickTarget[0], 0).setCursor(false, true);
    Editor._clickTarget.remove();
    Editor.fireEvent('contentChange');
  };

  // 给图片添加链接
  Editor._setImageLink = function(url){
    if (!Editor._clickTarget) {
      return;
    }
    var parent = Editor._clickTarget.parent();
    if (parent[0].tagName.toLocaleLowerCase() == 'a') {
      parent.attr('href', url).attr('_href', url);
      Editor.fireEvent('contentChange');
      return;
    }
    var html = Editor._clickTarget[0].outerHTML;
    Editor.selection.getRange().setStart(Editor._clickTarget[0], 0).setCursor(false, true);
    Editor._clickTarget.remove();
    Editor.execCommand('inserthtml', '<a href="' + url + '">' + html + '</a>');
  };

  // 删除图片链接
  Editor._removeImageLink = function(){
    if (!Editor._clickTarget) {
      return;
    }
    var parent = Editor._clickTarget.parent();
    if (parent[0].tagName.toLocaleLowerCase() == 'a') {
      var html = parent.html();
      Editor.selection.getRange().setStart(parent[0], 0).setCursor(false, true);
      parent.remove();
      Editor.execCommand('inserthtml', html);
    }
  };

  Editor._insertLinkText = function(text, link) {
    Editor.execCommand('inserthtml', '<a href="' + link + '">' + text + '</a>');
  };

  Editor._setTextLink = function(url){
    Editor.execCommand('link', {href: url});
  }

  Editor._saveRange = function(){
    var r = Editor.selection.getRange();
    Editor._tempRange = r.cloneRange();
  }

  Editor._resetRange = function(){
    if (Editor._tempRange) {
      var r = Editor.selection.getRange();
      var pre = Editor._tempRange;
      r.setStart(pre.startContainer, pre.startOffset)
      b.setEnd(pre.endContainer, pre.endOffset);
    }
  }

  Editor._imageFloat = function(align){
    if (!Editor._clickTarget) {
      return;
    }
    if (align == 'center') {
      Editor._clickTarget.css('float', 'none');
      Editor._clickTarget.parent().css('text-align', 'center');
    } else {
      Editor._clickTarget.css('float', align);
    }
    Editor.fireEvent('contentChange');
  }

  Editor._closeAttachEdit = function() {
    $('.tap-target-border').remove();
  }

  Editor._setEncodeContent = function(content){
    var con = content.replace(/wxb_dot/g, "'");
    Editor.setContent(con);
  }

  window.Editor = Editor;
});


$().ready(function(){

  var utils = baidu.editor.utils,
        uiUtils = baidu.editor.ui.uiUtils,
        UIBase = baidu.editor.ui.UIBase,
        domUtils = baidu.editor.dom.domUtils;
    var nodeStack = [];

  var editor = Editor;

  Editor.on('ready', function(e){
    console.log('ready:::::');
    CallApp('EditorReady', '1');
    
    setTimeout(function(){
      Editor.selection.getRange().setStart(document.getElementsByTagName('p')[0], 0).setCursor(false, true);
    }, 300);

    // touchend
    $('#JAPPEditor').on('tap', function(e) {
      var target = e.target;
      var tagName = target.tagName.toLocaleLowerCase();
      console.log('click tag:', target);
      var jqTarget = $(target);
      var data = {};
      if(tagName == 'qqmusic' || 
        (jqTarget.hasClass('xb_iframe_wrap') && jqTarget.find('.qqmusic_iframe').length) ||
        jqTarget.hasClass('qqmusic_iframe')) {
        data.type = 'music';
      } else if (tagName =='img') {
        data.type = 'image';
        var parent = jqTarget.parent();
        if (parent[0].tagName.toLocaleLowerCase() == 'a') {
          data.link = parent.attr('href');
        }
      } else if (tagName =='mpvoice' ||
        (jqTarget.hasClass('xb_iframe_wrap') && jqTarget.find('.audio_iframe').length) ||
        jqTarget.hasClass('audio_iframe')) {
        data.type = 'voice';
      } else if (jqTarget.hasClass('video_iframe_wrap')) {
        data.type = 'video';
      } else {
        data.type = 'text';
      }
      // && data.type != 'image'
      $('.tap-target-border').remove();
      if (data.type != 'text') {
        // e.stopPropagation();
        e.preventDefault();
        Editor.selection.getRange().setStartAfter(target).setCursor(false, true);
        var border = $('<div class="tap-target-border"></div>');
        var offset = jqTarget.offset();
        border.css({
          border: 'border: 1px dashed #f00',
          width: jqTarget.width(),
          height: jqTarget.height(),
          left: offset.left,
          top: offset.top
        });
        if (data.type != 'video') {
          $('body').append(border);
        }
      }
      // if (target.href) {
      //   e.preventDefault();
      // }
      // e.preventDefault();
      Editor._clickTarget = jqTarget;
      CallApp('ClickType', data);
    });
    

    Editor.addListener('contentChange',function(){
      // $('.tap-target-border').remove();
    });
  });

});

